@@ -28,6 +28,11 @@ def allBinaryTags(path=None):
     the release builds database.
     '''
     import json
+    try:  # pragma no cover
+        from urllib.request import urlopen
+        from urllib.error import URLError
+    except ImportError:  # pragma no cover
+        from urllib2 import urlopen, URLError
 
     try:
         if path is not None:
@@ -35,10 +40,9 @@ def allBinaryTags(path=None):
         elif os.path.exists(BINARY_TAGS_CACHE):
             btags = open(BINARY_TAGS_CACHE)
         else:
-            import urllib2
-            btags = urllib2.urlopen(BINARY_TAGS_URL)
+            btags = urlopen(BINARY_TAGS_URL)
         data = btags.read()
-    except (ImportError, urllib2.URLError, IOError):
+    except (ImportError, URLError, IOError):
         import pkg_resources
         data = pkg_resources.resource_string('LbPlatformUtils',
                                              'platforms-list.json')
@@ -62,8 +66,7 @@ def platform_info(all_binary_tags=None):
     if all_binary_tags is None:
         all_binary_tags = allBinaryTags()
 
-    if lpu.__version__:
-        info['LbPlatformUtils version'] = lpu.__version__
+    info['LbPlatformUtils version'] = lpu.__version__
 
     dirac_platform = lpu.dirac_platform()
 
@@ -170,7 +173,7 @@ def main(args=None):
     if args.flags:
         print('flags:', *info['flags'], sep='\n  - ')
 
-    if not args.no_platforms:
+    if not args.no_platforms:  # pragma no cover
         if info['compatible_binary_tags']:
             print(
                 'compatible_binary_tags:',
@@ -179,7 +182,7 @@ def main(args=None):
         else:
             print('compatible_binary_tags: []')
 
-    if info['container_technology']:
+    if info['container_technology']:  # pragma no cover
         if not args.no_platforms:
             print('container_technology:')
             for tech in info['container_technology']:
@@ -193,7 +196,7 @@ def main(args=None):
                 'container_technology:',
                 *info['container_technology'],
                 sep='\n  - ')
-    else:
+    else:  # pragma: no cover
         print('container_technology:', '[]' if args.no_platforms else '{}')
 
 
@@ -205,11 +208,33 @@ def host_binary_tag_script(args=None):
         import argparse
         parser = argparse.ArgumentParser(
             description='print default host binary tag string')
+        parser.add_argument(
+            '--maximum',
+            action='store_false',
+            help=
+            'use the highest know architecture the machine can run (default)')
+        parser.add_argument(
+            '--minimum',
+            action='store_true',
+            dest='minimum',
+            help='use baseline architecture rather than the highest known')
+        parser.set_defaults(minimum=False)
         args = parser.parse_args(args)
     except ImportError:  # pragma no cover
         import optparse
         parser = optparse.OptionParser(
             description='print default host binary tag string')
+        parser.add_option(
+            '--maximum',
+            action='store_false',
+            help=
+            'use the highest know architecture the machine can run (default)')
+        parser.add_option(
+            '--minimum',
+            action='store_true',
+            dest='minimum',
+            help='use baseline architecture rather than the highest known')
+        parser.set_defaults(minimum=False)
         args = parser.parse_args(args)[0]
     from LbPlatformUtils import host_binary_tag
-    print(host_binary_tag())
+    print(host_binary_tag(args.minimum))
