@@ -51,63 +51,87 @@ SINGULARITY_ROOTS = [
 def parse_os_release(file_obj):
     '''
     Extract OS id from content of /etc/os-release.
+
+    See https://www.freedesktop.org/software/systemd/man/os-release.html
     '''
     release = dict(
         stripped.split('=', 1) for line in file_obj
-        for stripped in (line.strip(), ) if stripped)
+        for stripped in (line.strip(), ) if '=' in stripped)
 
     for key in release:  # values might be surrounded by quotes
         release[key] = release[key].strip('"').strip("'")
 
-    if release['ID'] == 'opensuse-leap':
-        return 'suse' + release['VERSION_ID'].split('.', 1)[0]
-    elif release['ID'] in ('ubuntu', 'centos', 'debian', 'fedora'):
-        return release['ID'] + release['VERSION_ID'].replace('.', '')
-    elif release['ID'] == 'rhel':
-        if 'scientific' in release.get('ID_LIKE', ''):
-            release['ID'] = 'sl'
-        else:
-            release['ID'] = 'redhat'
-        return release['ID'] + release['VERSION_ID'].split('.', 1)[0]
+    name = release.get('ID', 'linux').split('-', 1)[0]
+    compatible = name + release.get('ID_LIKE', '')
+    if 'rhel' in compatible or 'suse' in compatible:
+        version = release.get('VERSION_ID', '').split('.', 1)[0]
+    else:
+        version = (release.get('VERSION_ID',
+                               'testing' if name == 'debian' else '').replace(
+                                   '.', ''))
+    if name == 'scientific':
+        name == 'sl'  # that's the traditional name of
 
-    raise ValueError('unrecognized os-release content')
+    return name, version
+
+
+def parse_system_release(s):
+    '''
+    Extract OS id from content of /etc/redhat-release like files.
+    '''
+    name = 'unknown'
+    version = ''
+
+    m = re.match(r'(.*) release (\d+)', s)
+    if m:
+        fullname, version = m.groups()
+        if 'CERN' in fullname:
+            name = 'slc'
+        elif 'Scientific Linux' in fullname:
+            name = 'sl'
+        elif 'CentOS' in fullname:
+            name = 'centos'
+        elif 'Red Hat Enterprise Linux' in fullname:
+            name = 'rhel'
+
+    return name, version
+
+
+def parse_lsb_release(lines):
+    '''
+    Extract OS id from content of /etc/lsb-release files (in Debian derived OSs).
+    '''
+    name = 'unknown'
+    version = ''
+
+    for l in lines:
+        if l.startswith('DISTRIB_ID='):
+            name = l.strip()[11:].lower()
+        elif l.startswith('DISTRIB_RELEASE='):
+            version = l.strip()[16:].replace('.', '')
+
+    return name, version
 
 
 def _Linux_os():
+    name = 'unknown'
+    version = ''
+
     try:
-        return parse_os_release(open('/etc/os-release'))
-    except (IOError, ValueError, KeyError):
-        pass  # os-release cannot be parsed, fall back on old style
-    dist = platform.linux_distribution(full_distribution_name=False)
-    dist_name = dist[0].lower()
-    dist_version = dist[1]
-    if dist_name in ('redhat', 'centos'):
-        rh_rel = open('/etc/%s-release' % dist_name).read().strip()
-        if 'CERN' in rh_rel:
-            dist_name = 'slc'
-        elif 'Scientific Linux' in rh_rel:
-            dist_name = 'sl'
-        dist_version = dist_version.split('.', 1)[0]
-    elif dist_name == 'suse':
-        dist_version = dist_version.split('.', 1)[0]
-    elif dist_name == 'debian':
-        dist_version = dist_version.split('.', 1)[0]
-        # there's a problem with vanilla Python not recognizing Ubuntu
-        # see https://sft.its.cern.ch/jira/browse/SPI-961
-        try:
-            for l in open('/etc/lsb-release'):
-                if l.startswith('DISTRIB_ID='):
-                    dist_name = l.strip()[11:].lower()
-                elif l.startswith('DISTRIB_RELEASE='):
-                    dist_version = l.strip()[16:]
-        except IOError:
-            pass  # lsb-release is missing
-    if dist_name == 'ubuntu':
-        dist_version = dist_version.replace('.', '')
-    elif dist_name == '':
-        # Fall back on a generic Linux build if distribution detection fails
-        dist_name = 'linux'
-    return dist_name + dist_version
+        if os.path.exists('/etc/os-release'):
+            name, version = parse_os_release(open('/etc/os-release'))
+
+        elif os.path.exists('/etc/redhat-release'):
+            name, version = parse_system_release(
+                open('/etc/redhat-release').read())
+
+        elif os.path.exists('/etc/lsb-release'):
+            name, version = parse_lsb_release(open('/etc/lsb-release'))
+
+    except IOError:  # pragma: no cover
+        pass  # ignore the error when we cannot read the file
+
+    return name + version
 
 
 def _Darwin_os():
@@ -126,18 +150,25 @@ def _unknown_os():
 os_id = globals().get('_%s_os' % platform.system(), _unknown_os)
 
 
-def architecture():
+def architecture(minimum=False):
     '''
     Return the host CPU architecture based on the supported instructions.
 
     The result is the most recent known architecture matching the supported
-    instructions.
+    instructions, unless minimum is set to True, in which case we return the
+    base architecture.
     '''
     from LbPlatformUtils.architectures import get_supported_archs
+    from itertools import dropwhile
+
     flags = microarch_flags()
     if flags:
         try:
-            return next(get_supported_archs(flags))
+            return next(
+                # if minimum is True, we pick only the last of the list
+                # stopping at 'x86_64' (we ignore i686 by convention)
+                dropwhile(lambda a: minimum and a != 'x86_64',
+                          get_supported_archs(flags)))
         except StopIteration:  # pragma no cover
             # FIXME: it's not really possible to get here because (currently)
             #        anything can run i686 code (according to the known
@@ -168,6 +199,7 @@ def microarch_flags():
         for l in cpuinfo:
             if l.startswith('flags'):
                 return set(l.split(':', 1)[1].split())
+    # FIXME: This should probably print a warning
     return set()
 
 
@@ -179,9 +211,9 @@ def compiler_id(cmd=os.environ.get('CC', 'cc')):
     env = dict(os.environ)
     env['LC_ALL'] = 'C'
     try:
-        m = re.search(r'(gcc|clang|icc|LLVM) version (\d+)\.(\d+)',
-                      check_output([cmd, '-v'],
-                                   stderr=STDOUT, env=env).decode('utf-8'))
+        m = re.search(
+            r'(gcc|clang|icc|LLVM) version (\d+)\.(\d+)',
+            check_output([cmd, '-v'], stderr=STDOUT, env=env).decode('utf-8'))
         comp = 'clang' if m.group(1) == 'LLVM' else m.group(1)
         vers = m.group(2)
         if (comp == 'gcc' and int(vers) < 7) or comp == 'clang':
@@ -201,5 +233,5 @@ def singularity_os_ids():
         return [(path, os_id) for path, os_id in SINGULARITY_ROOTS
                 if os.path.isdir(path)]
     except (CalledProcessError, OSError):
-        # ignore singlularity selftest failures (assume we cannot run it)
+        # ignore singularity selftest failures (assume we cannot run it)
         return []
