@@ -55,10 +55,10 @@ def test_os_id_function_selection():
     del sys.modules['LbPlatformUtils.inspect']
 
 
-def test_os_release_detection():
-    import platform
+def test_parse_os_release():
     import LbPlatformUtils.inspect as PU
     mappings = [
+        # docker run --rm centos:7 cat /etc/os-release
         ('centos7', '''NAME="CentOS Linux"
 VERSION="7 (Core)"
 ID="centos"
@@ -76,11 +76,12 @@ REDHAT_SUPPORT_PRODUCT="centos"
 REDHAT_SUPPORT_PRODUCT_VERSION="7"
 
 '''),
+        # docker run --rm ubuntu:latest cat /etc/os-release
         ('ubuntu1804', '''NAME="Ubuntu"
-VERSION="18.04.1 LTS (Bionic Beaver)"
+VERSION="18.04.2 LTS (Bionic Beaver)"
 ID=ubuntu
 ID_LIKE=debian
-PRETTY_NAME="Ubuntu 18.04.1 LTS"
+PRETTY_NAME="Ubuntu 18.04.2 LTS"
 VERSION_ID="18.04"
 HOME_URL="https://www.ubuntu.com/"
 SUPPORT_URL="https://help.ubuntu.com/"
@@ -89,7 +90,8 @@ PRIVACY_POLICY_URL="https://www.ubuntu.com/legal/terms-and-policies/privacy-poli
 VERSION_CODENAME=bionic
 UBUNTU_CODENAME=bionic
 '''),
-        ('suse15', '''NAME="openSUSE Leap"
+        # docker run --rm opensuse/leap cat /etc/os-release
+        ('opensuse15', '''NAME="openSUSE Leap"
 VERSION="15.0"
 ID="opensuse-leap"
 ID_LIKE="suse opensuse"
@@ -100,144 +102,120 @@ CPE_NAME="cpe:/o:opensuse:leap:15.0"
 BUG_REPORT_URL="https://bugs.opensuse.org"
 HOME_URL="https://www.opensuse.org/"
 '''),
-        ('sl7', '''NAME="Scientific Linux"
-VERSION="7.5 (Nitrogen)"
-ID="rhel"
-ID_LIKE="scientific centos fedora"
-VERSION_ID="7.5"
-PRETTY_NAME="Scientific Linux 7.5 (Nitrogen)"
+        # /cvmfs/cernvm-prod.cern.ch/cvm4/etc/os-release
+        ('scientific7', '''NAME="Scientific Linux"
+VERSION="7.6 (Nitrogen)"
+ID="scientific"
+ID_LIKE="rhel centos fedora"
+VERSION_ID="7.6"
+PRETTY_NAME="Scientific Linux 7.6 (Nitrogen)"
 ANSI_COLOR="0;31"
-CPE_NAME="cpe:/o:scientificlinux:scientificlinux:7.5:GA"
+CPE_NAME="cpe:/o:scientificlinux:scientificlinux:7.6:GA"
 HOME_URL="http://www.scientificlinux.org//"
 BUG_REPORT_URL="mailto:scientific-linux-devel@listserv.fnal.gov"
 
 REDHAT_BUGZILLA_PRODUCT="Scientific Linux 7"
-REDHAT_BUGZILLA_PRODUCT_VERSION=7.5
+REDHAT_BUGZILLA_PRODUCT_VERSION=7.6
 REDHAT_SUPPORT_PRODUCT="Scientific Linux"
-REDHAT_SUPPORT_PRODUCT_VERSION="7.5"'''),
+REDHAT_SUPPORT_PRODUCT_VERSION="7.6"'''),
+
+        # docker run --rm debian:stable cat /etc/os-release
+        ('debian9', '''PRETTY_NAME="Debian GNU/Linux 9 (stretch)"
+NAME="Debian GNU/Linux"
+VERSION_ID="9"
+VERSION="9 (stretch)"
+ID=debian
+HOME_URL="https://www.debian.org/"
+SUPPORT_URL="https://www.debian.org/support"
+BUG_REPORT_URL="https://bugs.debian.org/"
+'''),
+
+        # docker run --rm debian:testing cat /etc/os-release
+        ('debiantesting', '''PRETTY_NAME="Debian GNU/Linux buster/sid"
+NAME="Debian GNU/Linux"
+ID=debian
+HOME_URL="https://www.debian.org/"
+SUPPORT_URL="https://www.debian.org/support"
+BUG_REPORT_URL="https://bugs.debian.org/"
+'''),
+
         # this one is a guess
-        ('redhat7', '''ID="rhel"
+        ('rhel7', '''ID="rhel"
 VERSION_ID="7.5"'''),
+
+        # unidentified
+        ('linux', ''''''),
     ]
 
     for expected, os_release in mappings:
-        assert PU.parse_os_release(os_release.splitlines(True)) == expected, \
+        n, v = PU.parse_os_release(os_release.splitlines(True))
+        assert (n + v) == expected, \
             'tested os-release did not map to %s' % expected
 
-    try:
-        PU.parse_os_release(['ID=super-distro', 'VERSION_ID=1.0'])
-        assert False, 'ValueError exception expected'
-    except ValueError:
-        pass
 
-
-def test_rh_flavours():
-    import platform
+def test_parse_system_release():
     import LbPlatformUtils.inspect as PU
 
     mappings = [
         # RHEL 7 equivalent
-        (('centos', '7.4.1708', 'Core'),
-         ('/etc/centos-release', 'CentOS Linux release 7.4.1708 (Core)\n'),
-         'centos7'),
-        (('redhat', '7.4.1708', 'Core'),
-         ('/etc/redhat-release', 'Scientific Linux SL release 7.x\n'),
-         'sl7'),
-        (('redhat', '7.4.1708', 'Core'),
-         ('/etc/redhat-release', 'Redhat Enterprise Linux release 7.x\n'),
-         'redhat7'),
+        ('CentOS Linux release 7.4.1708 (Core)\n', 'centos7'),
+        ('Scientific Linux SL release 7.x\n', 'sl7'),
+        ('Red Hat Enterprise Linux release 7.x\n', 'rhel7'),
 
         # RHEL 6 equivalent
-        (('centos', '6.9', 'Core'),
-         ('/etc/centos-release', 'CentOS Linux release 6.x (Core)\n'),
-         'centos6'),
-        (('redhat', '6.9', 'Core'),
-         ('/etc/redhat-release', 'Scientific Linux SL release 6.x\n'),
-         'sl6'),
-        (('redhat', '6.9', 'Core'),
-         ('/etc/redhat-release', 'Scientific Linux CERN SLC release 6.x\n'),
-         'slc6'),
-        (('redhat', '6.9', 'Core'),
-         ('/etc/redhat-release', 'Redhat Enterprise Linux release 6.x\n'),
-         'redhat6'),
+        ('CentOS Linux release 6.x (Core)\n', 'centos6'),
+        ('Scientific Linux SL release 6.x\n', 'sl6'),
+        ('Scientific Linux CERN SLC release 6.x\n', 'slc6'),
+        ('Red Hat Enterprise Linux release 6.x\n', 'rhel6'),
 
-        # RHEL 7 equivalent
-        (('centos', '5.0', 'X'),
-         ('/etc/centos-release', 'CentOS Linux release 5.x (Core)\n'),
-         'centos5'),
-        (('redhat', '5.11', 'Boron'),
-         ('/etc/redhat-release', 'Scientific Linux SL release 5.x\n'),
-         'sl5'),
-        (('redhat', '5.11', 'Boron'),
-         ('/etc/redhat-release', 'Scientific Linux CERN SLC release 5.x\n'),
-         'slc5'),
-        (('redhat', '5.11', 'Boron'),
-         ('/etc/redhat-release', 'Redhat Enterprise Linux release 5.x\n'),
-         'redhat5'),
+        # RHEL 5 equivalent
+        ('CentOS Linux release 5.x (Core)\n', 'centos5'),
+        ('Scientific Linux SL release 5.x\n', 'sl5'),
+        ('Scientific Linux CERN SLC release 5.x\n', 'slc5'),
+        ('Red Hat Enterprise Linux release 5.x\n', 'rhel5'),
     ]
 
-    for platform._linux_dist_short, (relname, reldata), expected in mappings:
-        open._overrides[relname] = reldata
-        open._overrides['/etc/os-release'] = None
-        assert PU._Linux_os() == expected, \
-            '%s did not map to %s' % (reldata.strip(), expected)
+    for reldata, expected in mappings:
+        n, v = PU.parse_system_release(reldata)
+        assert (n + v) == expected, \
+            '%r did not map to %s' % (reldata.strip(), expected)
 
 
-def test_suse():
-    import platform
-    import LbPlatformUtils.inspect as PU
-
-    mappings = [
-        (('suse', '9.9.9', 'xyz'), 'suse9'),
-    ]
-
-    for platform._linux_dist_short, expected in mappings:
-        assert PU._Linux_os() == expected, \
-            '%s did not map to %s' % (platform._linux_dist_short, expected)
-
-
-def test_debian_flavours():
-    import platform
+def test_parse_lsb_release():
     import LbPlatformUtils.inspect as PU
 
     mappings = [
         # Ubuntu (native)
-        (('Ubuntu', '16.04', 'xenial'),
-         ('/etc/lsb-release', 'DISTRIB_ID=Ubuntu\nDISTRIB_RELEASE=16.04\n'),
-         'ubuntu1604'),
-        (('Ubuntu', '18.04', 'bionic'),
-         ('/etc/lsb-release', 'DISTRIB_ID=Ubuntu\nDISTRIB_RELEASE=18.04\n'),
-         'ubuntu1804'),
-
-        # Ubuntu (vanilla Python)
-        (('debian', 'stretch/sid', ''),
-         ('/etc/lsb-release', 'DISTRIB_ID=Ubuntu\nDISTRIB_RELEASE=16.04\n'),
-         'ubuntu1604'),
-        (('debian', 'jessie/sid', ''),
-         ('/etc/lsb-release', 'DISTRIB_ID=Ubuntu\nDISTRIB_RELEASE=18.04\n'),
-         'ubuntu1804'),
-
-        # Debian
-        (('debian', '9.3', ''),
-         ('/etc/lsb-release', None),
-         'debian9'),
-        (('debian', '8.10', ''),
-         ('/etc/lsb-release', None),
-         'debian8'),
-        (('debian', '7.11', ''),
-         ('/etc/lsb-release', None),
-         'debian7'),
-
-        # Unknown by Python platform module
-        (('', '', ''),
-         ('/etc/lsb-release', None),
-         'linux'),
+        ('DISTRIB_ID=Ubuntu\nDISTRIB_RELEASE=16.04\n', 'ubuntu1604'),
+        ('DISTRIB_ID=Ubuntu\nDISTRIB_RELEASE=18.04\n', 'ubuntu1804'),
     ]
 
-    for platform._linux_dist_short, (relname, reldata), expected in mappings:
+    for reldata, expected in mappings:
+        n, v = PU.parse_lsb_release(reldata.splitlines(True))
+        assert (n + v) == expected, \
+            '%r did not map to %s' % (reldata.strip(), expected)
+
+
+def test_linux_dispatching():
+    import LbPlatformUtils.inspect as PU
+
+    mappings = [
+        (('/etc/os-release', 'ID="rhel"\nVERSION_ID="7.5"\n'), 'rhel7'),
+        (('/etc/redhat-release', 'Scientific Linux CERN SLC release 6.x\n'),
+         'slc6'),
+        (('/etc/lsb-release', 'DISTRIB_ID=Ubuntu\nDISTRIB_RELEASE=18.04\n'),
+         'ubuntu1804'),
+        (('/etc/dummy-release', None), 'unknown'),
+    ]
+
+    for (relname, reldata), expected in mappings:
+        open._overrides['/etc/os-release'] = None
+        open._overrides['/etc/redhat-release'] = None
+        open._overrides['/etc/lsb-release'] = None
         open._overrides[relname] = reldata
-        assert PU._Linux_os() == expected, \
-            '%s did not map to %s' % (platform._linux_dist_short, expected)
+        found = PU._Linux_os()
+        assert found == expected, \
+            '%s did not map to %s' % (relname, expected)
 
 
 def test_macos():
@@ -286,17 +264,15 @@ def test_host_os():
         yield
         PUI.os_id = orig_os_id
 
+    # test for highest matching
     mappings = [
         ('x86_64', ('debian8', 'Linux', flags_for_arch('x86_64')),
          'x86_64-debian8'),
-        ('', ('slc6', 'Linux', 'nothing'),
-         'unknown-slc6'),
-        ('x86_64', ('slc6', 'Linux', flags_for_arch('x86_64')),
-         'x86_64-slc6'),
+        ('', ('slc6', 'Linux', 'nothing'), 'unknown-slc6'),
+        ('x86_64', ('slc6', 'Linux', flags_for_arch('x86_64')), 'x86_64-slc6'),
         ('x86_64', ('centos7', 'Linux', flags_for_arch('haswell')),
          'haswell-centos7'),
-        ('x86_64', ('suse42', 'Linux', 'nothing'),
-         'x86_64-centos7'),
+        ('x86_64', ('opensuse42', 'Linux', 'nothing'), 'x86_64-centos7'),
     ]
     for platform._machine, \
             (os_id, platform._system, open._overrides['/proc/cpuinfo']), \
@@ -305,6 +281,23 @@ def test_host_os():
             detected = PU.host_os()
             assert detected == expected
 
+    # test for baseline
+    mappings = [
+        ('x86_64', ('debian8', 'Linux', flags_for_arch('x86_64')),
+         'x86_64-debian8'),
+        ('', ('slc6', 'Linux', 'nothing'), 'unknown-slc6'),
+        ('x86_64', ('slc6', 'Linux', flags_for_arch('x86_64')), 'x86_64-slc6'),
+        ('x86_64', ('centos7', 'Linux', flags_for_arch('haswell')),
+         'x86_64-centos7'),
+        ('x86_64', ('opensuse42', 'Linux', 'nothing'), 'x86_64-centos7'),
+    ]
+    for platform._machine, \
+            (os_id, platform._system, open._overrides['/proc/cpuinfo']), \
+            expected in mappings:
+        with force_os_id(os_id):
+            detected = PU.host_os(minimum=True)
+            assert detected == expected
+
 
 def test_microarch_flags():
     import LbPlatformUtils.inspect as PUI
@@ -349,30 +342,27 @@ def test_dirac_platform():
     mappings = [
         ('x86_64', ('debian8', 'Linux', flags_for_arch('x86_64')),
          'x86_64-unknown'),
-        ('', ('slc6', 'Linux', 'dummy'),
-         'unknown-slc6'),
-        ('x86_64', ('slc6', 'Linux', flags_for_arch('x86_64')),
-         'x86_64-slc6'),
+        ('', ('slc6', 'Linux', 'dummy'), 'unknown-slc6'),
+        ('x86_64', ('slc6', 'Linux', flags_for_arch('x86_64')), 'x86_64-slc6'),
         ('x86_64', ('centos7', 'Linux', flags_for_arch('core2')),
          'core2-centos7'),
-        ('x86_64', ('centos7', 'Linux', flags_for_arch('nehalem') + ' abc xyz'),
+        ('x86_64', ('centos7', 'Linux',
+                    flags_for_arch('nehalem') + ' abc xyz'),
          'nehalem-centos7'),
         ('x86_64', ('centos7', 'Linux', flags_for_arch('sandybridge')),
          'sandybridge-centos7'),
-        ('x86_64', ('centos7', 'Linux', flags_for_arch('sandybridge') + ' avx2'),
+        ('x86_64', ('centos7', 'Linux',
+                    flags_for_arch('sandybridge') + ' avx2'),
          'sandybridge-centos7'),
         ('x86_64', ('centos7', 'Linux', flags_for_arch('haswell')),
          'haswell-centos7'),
-        ('x86_64', ('suse42', 'Linux', flags_for_arch('x86_64')),
+        ('x86_64', ('opensuse42', 'Linux', flags_for_arch('x86_64')),
          'x86_64-centos7'),
-        ('x86_64', ('redhat6', 'Linux', flags_for_arch('x86_64')),
+        ('x86_64', ('rhel6', 'Linux', flags_for_arch('x86_64')),
          'x86_64-slc6'),
-        ('x86_64', ('slc5', 'NoLinux', None),
-         'x86_64-slc5'),
-        ('i386', ('slc5', 'NoLinux', None),
-         'i386-slc5'),
-        ('i686', ('slc5', 'NoLinux', None),
-         'i686-slc5'),
+        ('x86_64', ('slc5', 'NoLinux', None), 'x86_64-slc5'),
+        ('i386', ('slc5', 'NoLinux', None), 'i386-slc5'),
+        ('i686', ('slc5', 'NoLinux', None), 'i686-slc5'),
     ]
     for platform._machine, \
             (os_id, platform._system, open._overrides['/proc/cpuinfo']), \
@@ -382,6 +372,11 @@ def test_dirac_platform():
             detected = PU.dirac_platform()
             assert detected == expected
 
+    # Check dirac_platform detection override
+    open._overrides['/proc/cpuinfo'] = flags_for_arch('x86_64')
+    assert PU.can_run(
+        PU.dirac_platform(force_os='sl7'), 'x86_64-centos7')
+
 
 def test_compiler_id():
     import subprocess
@@ -400,46 +395,30 @@ def test_compiler_id():
             if kwargs.get('stderr') == subprocess.STDOUT:
                 return (stdout + stderr).encode('utf-8')
             return stdout.encode('utf-8')
+
         check_output_orig = PUI.check_output
         PUI.check_output = mock_check_output
         yield
         PUI.check_output = check_output_orig
 
-    mappings = [
-        (('',
-          'blah\nThread model: posix\n'
-          'gcc version 5.4.0 20160609 (Ubuntu 5.4.0-6ubuntu1~16.04.6)',
-          0), 'gcc54'),
-        (('',
-          'blah\nThread model: posix\n'
-          'gcc version 4.4.7 20120313 (Red Hat 4.4.7-18) (GCC)',
-          0), 'gcc44'),
-        (('',
-          'blah\nThread model: posix\n'
-          'gcc version 4.8.5 20150623 (Red Hat 4.8.5-16) (GCC)',
-          0), 'gcc48'),
-        (('',
-          'blah\nThread model: posix\n'
-          'gcc version 6.2.0 (GCC)',
-          0), 'gcc62'),
-        (('',
-          'blah\nThread model: posix\n'
-          'gcc version 7.3.0 (GCC)',
-          0), 'gcc7'),
-        (('',
-          'clang version 5.0.0 (trunk 296300)\n'
-          'Target: x86_64-unknown-linux-gnu',
-          0), 'clang50'),
-        (('',
-          'icc version 18.0.0 (gcc version 4.4.7 compatibility)\n',
-          0), 'icc18'),
-        (('',
-          'new fancy stuff 10.0\n',
-          0), 'unknown'),
-        (('',
-          '-bash: cc: command not found\n',
-          127), 'unknown')
-    ]
+    mappings = [(('', 'blah\nThread model: posix\n'
+                  'gcc version 5.4.0 20160609 (Ubuntu 5.4.0-6ubuntu1~16.04.6)',
+                  0), 'gcc54'),
+                (('', 'blah\nThread model: posix\n'
+                  'gcc version 4.4.7 20120313 (Red Hat 4.4.7-18) (GCC)', 0),
+                 'gcc44'),
+                (('', 'blah\nThread model: posix\n'
+                  'gcc version 4.8.5 20150623 (Red Hat 4.8.5-16) (GCC)', 0),
+                 'gcc48'),
+                (('', 'blah\nThread model: posix\n'
+                  'gcc version 6.2.0 (GCC)', 0), 'gcc62'),
+                (('', 'blah\nThread model: posix\n'
+                  'gcc version 7.3.0 (GCC)', 0), 'gcc7'),
+                (('', 'clang version 5.0.0 (trunk 296300)\n'
+                  'Target: x86_64-unknown-linux-gnu', 0), 'clang50'),
+                (('', 'icc version 18.0.0 (gcc version 4.4.7 compatibility)\n',
+                  0), 'icc18'), (('', 'new fancy stuff 10.0\n', 0), 'unknown'),
+                (('', '-bash: cc: command not found\n', 127), 'unknown')]
     for (stdout, stderr, returncode), expected in mappings:
         with override_check_output(stdout, stderr, returncode):
             assert PUI.compiler_id() == expected
